package com.fuyuanshen.modules.system.rest;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.RequiredArgsConstructor;
import com.fuyuanshen.annotation.Log;
import com.fuyuanshen.modules.system.domain.DeviceType;
import com.fuyuanshen.modules.system.domain.dto.DeviceTypeQueryCriteria;
import com.fuyuanshen.modules.system.service.DeviceTypeService;
import com.fuyuanshen.modules.utils.ResponseVO;
import com.fuyuanshen.utils.PageResult;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/13
 **/
@Api(tags = "设备：设备类型管理")
@RestController
@RequiredArgsConstructor
@RequestMapping("/api/deviceType")
public class DeviceTypeController {

    private final DeviceTypeService deviceTypeService;


    @GetMapping
    @ApiOperation("分页查询设备类型")
    public ResponseVO<PageResult<DeviceType>> queryDeviceType(DeviceTypeQueryCriteria criteria) {
        Page<Object> page = new Page<>(criteria.getPage(), criteria.getSize());
        return ResponseVO.success(deviceTypeService.queryAll(criteria, page));
    }


    @GetMapping(value = "/all")
    @ApiOperation("查询所有设备类型")
    public ResponseVO<Object> queryDeviceTypes() {
        List<DeviceType> deviceTypes = deviceTypeService.queryDeviceTypes();
        return ResponseVO.success(deviceTypes);
    }


    @Log("新增设备类型")
    @ApiOperation("新增设备类型")
    @PostMapping(value = "/add")
    public ResponseVO<Object> createDeviceType(@Validated @RequestBody DeviceType resources) {
        deviceTypeService.create(resources);
        return ResponseVO.success(null);
    }


    @Log("修改设备类型")
    @ApiOperation("修改设备类型")
    @PutMapping(value = "/update")
    public ResponseVO<Object> updateDeviceType(@Validated @RequestBody DeviceType resources) {
        deviceTypeService.update(resources);
        return ResponseVO.success(null);
    }


    @Log("删除设备类型")
    @ApiOperation("删除设备类型")
    @DeleteMapping(value = "/delete")
    public ResponseEntity<Object> deleteDeviceType(@ApiParam(value = "传ID数组[]") @RequestBody List<Long> ids) {
        deviceTypeService.deleteAll(ids);
        return new ResponseEntity<>(HttpStatus.OK);
    }


    @GetMapping(value = "/communicationMode")
    @ApiOperation("获取设备类型通讯方式")
    public ResponseVO<DeviceType> getCommunicationMode(@ApiParam(value = "设备类型ID", required = true) Long id) {
        return ResponseVO.success(deviceTypeService.getById(id));
    }


}
