package com.fuyuanshen.modules.system.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import com.fuyuanshen.modules.system.domain.DeviceLog;
import com.fuyuanshen.modules.system.domain.dto.DeviceLogQueryCriteria;
import com.fuyuanshen.modules.system.mapper.DeviceLogMapper;
import com.fuyuanshen.modules.system.service.DeviceLogService;
import com.fuyuanshen.utils.FileUtil;
import com.fuyuanshen.utils.PageResult;
import com.fuyuanshen.utils.PageUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author: WY
 * @Date: 2025/5/24
 **/
@Service
@RequiredArgsConstructor
public class DeviceLogServiceImpl extends ServiceImpl<DeviceLogMapper, DeviceLog> implements DeviceLogService {

    private final DeviceLogMapper deviceLogMapper;

    @Override
    public PageResult<DeviceLog> queryAll(DeviceLogQueryCriteria criteria, Page<Object> page){
        return PageUtil.toPage(deviceLogMapper.findAll(criteria, page));
    }

    @Override
    public List<DeviceLog> queryAll(DeviceLogQueryCriteria criteria){
        return deviceLogMapper.findAll(criteria);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void create(DeviceLog resources) {
        deviceLogMapper.insert(resources);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void update(DeviceLog resources) {
        DeviceLog deviceLog = getById(resources.getId());
        deviceLog.copy(resources);
        deviceLogMapper.updateById(deviceLog);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteAll(List<Long> ids) {
        deviceLogMapper.deleteBatchIds(ids);
    }

    @Override
    public void download(List<DeviceLog> all, HttpServletResponse response) throws IOException {
        List<Map<String, Object>> list = new ArrayList<>();
        for (DeviceLog deviceLog : all) {
            Map<String,Object> map = new LinkedHashMap<>();
            map.put("设备行为", deviceLog.getDeviceAction());
            map.put("设备名称", deviceLog.getDeviceName());
            map.put("数据来源", deviceLog.getDataSource());
            map.put("内容", deviceLog.getContent());
            map.put("创建时间", deviceLog.getCreateTime());
            list.add(map);
        }
        FileUtil.downloadExcel(list, response);
    }

}
