/*
 *  Copyright 2019-2025 Zheng Jie
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package com.fuyuanshen.modules.system.service.impl.app;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuyuanshen.exception.BadRequestException;
import com.fuyuanshen.modules.system.domain.Role;
import com.fuyuanshen.modules.system.domain.app.APPUser;
import com.fuyuanshen.modules.system.domain.dto.UserQueryCriteria;
import com.fuyuanshen.modules.system.domain.dto.app.APPUserDTO;
import com.fuyuanshen.modules.system.mapper.app.APPUserMapper;
import com.fuyuanshen.modules.system.service.app.APPUserService;
import com.fuyuanshen.modules.utils.ResponseVO;
import com.fuyuanshen.utils.*;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Set;

/**
 * @author Zheng Jie
 * @date 2018-11-23
 */
@Service
@RequiredArgsConstructor
public class APPUserServiceImpl extends ServiceImpl<APPUserMapper, APPUser> implements APPUserService {

    private final APPUserMapper appUserMapper;



    /**
     * 查询APP/小程序用户
     *
     * @param criteria
     * @param page
     * @return
     */
    @Override
    public PageResult<APPUser> queryAPPUser(UserQueryCriteria criteria, Page<APPUser> page) {
        Page<APPUser> appUserPage = appUserMapper.queryAPPUser(page, criteria);
        PageResult<APPUser> pageResult = new PageResult<>();
        pageResult.setContent(appUserPage.getRecords());
        pageResult.setTotalElements(appUserPage.getTotal());
        return pageResult ;
    }

    @Override
    public APPUser getLoginData(String userName) {
        return appUserMapper.appFindByUsername(userName);
    }

    @Override
    public ResponseVO<Object> addUser(APPUserDTO user) {

        String username = user.getPhoneNumber().toString();
        if (appUserMapper.getByUsername(username) != null) {
            throw new BadRequestException("该手机号已被注册");
        }
                APPUser appUser = new APPUser();
                appUser.setUsername(user.getPhoneNumber().toString());
                appUser.setPassword(user.getPassword());
                appUser.setNickName(user.getPhoneNumber());
                appUser.setUserLevel((byte) 1);
                appUser.setPhone(Long.valueOf(user.getPhoneNumber()));
                appUser.setAdmin((byte) 1);
                appUser.setEnabled(true);
                appUser.setUserType(0);
                System.out.println("--------------------"+appUser);


                appUserMapper.insert(appUser);
                return ResponseVO.success("注册成功");

    }

    @Override
    public Integer selectRoleByUserLevel(Set<Role> roles) {
        return 0;
    }

}
